@@ -151,13 +151,12 @@ configFromConfigMonoid configStackRoot configUserConfigPath mresolver mproject c
 
      configDocker <-
          dockerOptsFromMonoid (fmap fst mproject) configStackRoot mresolver configMonoidDockerOpts
-     configNix <- nixOptsFromMonoid (fmap fst mproject) configStackRoot configMonoidNixOpts
+     configNix <- nixOptsFromMonoid (fmap fst mproject) mresolver configMonoidNixOpts os
 
      rawEnv <- liftIO getEnvironment
-     origEnv <- mkEnvOverride configPlatform
-              $ augmentPathMap (map toFilePath configMonoidExtraPath)
-              $ Map.fromList
-              $ map (T.pack *** T.pack) rawEnv
+     pathsEnv <- augmentPathMap (map toFilePath configMonoidExtraPath)
+                                (Map.fromList (map (T.pack *** T.pack) rawEnv))
+     origEnv <- mkEnvOverride configPlatform pathsEnv
      let configEnvOverride _ = return origEnv
 
      platformOnlyDir <- runReaderT platformOnlyRelDir (configPlatform,configPlatformVariant)
@@ -204,6 +203,7 @@ configFromConfigMonoid configStackRoot configUserConfigPath mresolver mproject c
          configRebuildGhcOptions = fromMaybe False configMonoidRebuildGhcOptions
          configApplyGhcOptions = fromMaybe AGOLocals configMonoidApplyGhcOptions
          configAllowNewer = fromMaybe False configMonoidAllowNewer
+         configDefaultTemplate = configMonoidDefaultTemplate
 
      return Config {..}
 
@@ -277,7 +277,7 @@ loadConfig :: (MonadLogger m,MonadIO m,MonadCatch m,MonadThrow m,MonadBaseContro
            -- ^ Config monoid from parsed command-line arguments
            -> Maybe (Path Abs File)
            -- ^ Override stack.yaml
-           -> Maybe (AbstractResolver)
+           -> Maybe AbstractResolver
            -- ^ Override resolver
            -> m (LoadConfig m)
 loadConfig configArgs mstackYaml mresolver = do
@@ -477,7 +477,8 @@ resolvePackageLocation menv projRoot (PLRemote url remotePackageType) = do
                     Nothing
                 readInNull dirTmp commandName menv
                     (resetCommand ++ [T.unpack commit])
-                    Nothing
+                    (Just $ "Please ensure that commit " <> commit <>
+                      " exists within " <> url)
 
         case remotePackageType of
             RPTHttpTarball -> do
