@@ -4,6 +4,7 @@
 {-# LANGUAGE TupleSections #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE CPP #-}
 
 -- | Run a GHCi configured with the user's package(s).
 
@@ -52,6 +53,10 @@ import           Stack.Types
 import           Stack.Types.Internal
 import           System.Directory (getTemporaryDirectory)
 
+#ifndef WINDOWS
+import qualified System.Posix.Files as Posix
+#endif
+
 -- | Command-line options for GHC.
 data GhciOpts = GhciOpts
     { ghciNoBuild            :: !Bool
@@ -61,6 +66,7 @@ data GhciOpts = GhciOpts
     , ghciAdditionalPackages :: ![String]
     , ghciMainIs             :: !(Maybe Text)
     , ghciSkipIntermediate   :: !Bool
+    , ghciHidePackages       :: !Bool
     , ghciBuildOpts          :: !BuildOpts
     } deriving Show
 
@@ -93,8 +99,11 @@ ghci GhciOpts{..} = do
     mainFile <- figureOutMainFile bopts mainIsTargets targets pkgs
     wc <- getWhichCompiler
     let pkgopts = hidePkgOpt ++ genOpts ++ ghcOpts
-        hidePkgOpt = if null pkgs then [] else ["-hide-all-packages"]
-        genOpts = nubOrd (concatMap (concatMap (bioOneWordOpts . snd) . ghciPkgOpts) pkgs)
+        hidePkgOpt = if null pkgs || not ghciHidePackages then [] else ["-hide-all-packages"]
+        oneWordOpts bio
+            | ghciHidePackages = bioOneWordOpts bio ++ bioPackageFlags bio
+            | otherwise = bioOneWordOpts bio
+        genOpts = nubOrd (concatMap (concatMap (oneWordOpts . snd) . ghciPkgOpts) pkgs)
         (omittedOpts, ghcOpts) = partition badForGhci $
             concatMap (concatMap (bioOpts . snd) . ghciPkgOpts) pkgs ++
             getUserOptions Nothing ++
@@ -139,6 +148,7 @@ ghci GhciOpts{..} = do
                     loadModules = ":load " <> unwords (map show thingsToLoad)
                     bringIntoScope = ":module + " <> unwords modulesToLoad
                 liftIO (writeFile fp (unlines [loadModules,bringIntoScope]))
+                setScriptPerms fp
                 execGhci (macrosOpts ++ ["-ghci-script=" <> fp])
 
 -- | Figure out the main-is file to load based on the targets. Sometimes there
@@ -438,8 +448,22 @@ getIntermediateDeps sourceMap targets =
 
 preprocessCabalMacros :: MonadIO m => [GhciPkgInfo] -> Path Abs File -> m [String]
 preprocessCabalMacros pkgs out = liftIO $ do
-    let fps = nubOrd (concatMap (catMaybes . map (bioCabalMacros . snd) . ghciPkgOpts) pkgs)
+    let fps = nubOrd (concatMap (mapMaybe (bioCabalMacros . snd) . ghciPkgOpts) pkgs)
     files <- mapM (S8.readFile . toFilePath) fps
     if null files then return [] else do
         S8.writeFile (toFilePath out) $ S8.intercalate "\n#undef CURRENT_PACKAGE_KEY\n" files
         return ["-optP-include", "-optP" <> toFilePath out]
+
+setScriptPerms :: MonadIO m => FilePath -> m ()
+#ifdef WINDOWS
+setScriptPerms _ = do
+    return ()
+#else
+setScriptPerms fp = do
+    liftIO $ Posix.setFileMode fp $ foldl1 Posix.unionFileModes
+        [ Posix.ownerReadMode
+        , Posix.ownerWriteMode
+        , Posix.groupReadMode
+        , Posix.otherReadMode
+        ]
+#endif
