@@ -1,14 +1,19 @@
 {-# LANGUAGE ConstraintKinds #-}
+{-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts #-}
-{-# LANGUAGE OverloadedStrings, TemplateHaskell #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE TemplateHaskell #-}
 
 -- | Run commands in a nix-shell
 module Stack.Nix
   (reexecWithOptionalShell
   ,nixCmdName
+  ,nixHelpOptName
   ) where
 
 import           Control.Applicative
+import           Control.Arrow ((***))
+import           Control.Exception (Exception,throw)
 import           Control.Monad
 import           Control.Monad.Catch (try,MonadCatch)
 import           Control.Monad.IO.Class (MonadIO,liftIO)
@@ -17,15 +22,19 @@ import           Control.Monad.Reader (MonadReader,asks)
 import           Control.Monad.Trans.Control (MonadBaseControl)
 import           Data.Char (toUpper)
 import           Data.List (intercalate)
+import           Data.Traversable
 import           Data.Maybe
 import           Data.Monoid
 import           Data.Streaming.Process (ProcessExitedUnsuccessfully(..))
 import qualified Data.Text as T
 import           Data.Version (showVersion)
+import           Data.Typeable (Typeable)
 import           Network.HTTP.Client.Conduit (HasHttpManager)
+import           Path
+import           Path.IO
 import qualified Paths_stack as Meta
 import           Prelude -- Fix redundant import warnings
-import           Stack.Constants (stackProgName)
+import           Stack.Constants (stackProgName,platformVariantEnvVar)
 import           Stack.Docker (reExecArgName)
 import           Stack.Exec (exec)
 import           System.Process.Read (getEnvOverride)
@@ -39,15 +48,16 @@ import           System.Exit (exitSuccess, exitWith)
 -- Otherwise, runs the inner action.
 reexecWithOptionalShell
     :: M env m
-    => IO ()
+    => Maybe (Path Abs Dir)
+    -> IO ()
     -> m ()
-reexecWithOptionalShell inner =
+reexecWithOptionalShell mprojectRoot inner =
   do config <- asks getConfig
      inShell <- getInShell
      isReExec <- asks getReExec
      if nixEnable (configNix config) && not inShell && not isReExec
-       then runShellAndExit getCmdArgs
-       else liftIO (inner >> exitSuccess)
+       then runShellAndExit mprojectRoot getCmdArgs
+       else liftIO inner
   where
     getCmdArgs = do
         args <-
@@ -57,22 +67,28 @@ reexecWithOptionalShell inner =
         exePath <- liftIO getExecutablePath
         return (exePath, args)
 
-runShellAndExit :: M env m
-                => m (String, [String])
-                -> m ()
-runShellAndExit getCmdArgs = do
+runShellAndExit
+    :: M env m
+    => Maybe (Path Abs Dir)
+    -> m (String, [String])
+    -> m ()
+runShellAndExit mprojectRoot getCmdArgs = do
      config <- asks getConfig
      envOverride <- getEnvOverride (configPlatform config)
-     (cmnd,args) <- getCmdArgs
-     let mshellFile = nixInitFile (configNix config)
-         pkgsInConfig = nixPackages (configNix config)
+     (cmnd,args) <- fmap (escape *** map escape) getCmdArgs
+     mshellFile <-
+         traverse (resolveFile (fromMaybeProjectRoot mprojectRoot)) $
+         nixInitFile (configNix config)
+     let pkgsInConfig = nixPackages (configNix config)
+         pureShell = nixPureShell (configNix config)
          nixopts = case mshellFile of
-           Just filePath -> [filePath]
+           Just fp -> [toFilePath fp]
            Nothing -> ["-E", T.unpack $ T.intercalate " " $ concat
                               [["with (import <nixpkgs> {});"
                                ,"runCommand \"myEnv\" {"
                                ,"buildInputs=lib.optional stdenv.isLinux glibcLocales ++ ["],pkgsInConfig,["];"
-                               ,T.pack inShellEnvVar,"=1 ;"
+                               ,T.pack platformVariantEnvVar <> "=''nix'';"
+                               ,T.pack inShellEnvVar <> "=1;"
                                ,"STACK_IN_NIX_EXTRA_ARGS=''"]
                                ,      (map (\p -> T.concat
                                                   ["--extra-lib-dirs=${",p,"}/lib"
@@ -80,15 +96,15 @@ runShellAndExit getCmdArgs = do
                                            pkgsInConfig), ["'' ;"
                                ,"} \"\""]]]
                     -- glibcLocales is necessary on Linux to avoid warnings about GHC being incapable to set the locale.
-         fullArgs = concat [ -- ["--pure"],
+         fullArgs = concat [if pureShell then ["--pure"] else [],
                             map T.unpack (nixShellOptions (configNix config))
                            ,nixopts
-                           ,["--command", intercalate " " (map escape (cmnd:args))
-                                          ++ " $STACK_IN_NIX_EXTRA_ARGS"]
+                           ,["--command", intercalate " " (cmnd:"$STACK_IN_NIX_EXTRA_ARGS":args)]
                            ]
+
      $logDebug $
          "Using a nix-shell environment " <> (case mshellFile of
-            Just filePath -> "from file: " <> (T.pack filePath)
+            Just path -> "from file: " <> (T.pack (toFilePath path))
             Nothing -> "with nix packages: " <> (T.intercalate ", " pkgsInConfig))
      e <- try (exec envOverride "nix-shell" fullArgs)
      case e of
@@ -102,6 +118,10 @@ escape str = "'" ++ foldr (\c -> if c == '\'' then
                                  else (c:)) "" str
                  ++ "'"
 
+-- | Fail with friendly error if project root not set.
+fromMaybeProjectRoot :: Maybe (Path Abs Dir) -> Path Abs Dir
+fromMaybeProjectRoot = fromMaybe (throw CannotDetermineProjectRootException)
+
 -- | 'True' if we are currently running inside a Nix.
 getInShell :: (MonadIO m) => m Bool
 getInShell = liftIO (isJust <$> lookupEnv inShellEnvVar)
@@ -116,6 +136,21 @@ inShellEnvVar = concat [map toUpper stackProgName,"_IN_NIXSHELL"]
 nixCmdName :: String
 nixCmdName = "nix"
 
+nixHelpOptName :: String
+nixHelpOptName = nixCmdName ++ "-help"
+
+-- | Exceptions thrown by "Stack.Nix".
+data StackNixException
+  = CannotDetermineProjectRootException
+    -- ^ Can't determine the project root (location of the shell file if any).
+  deriving (Typeable)
+
+instance Exception StackNixException
+
+instance Show StackNixException where
+  show CannotDetermineProjectRootException =
+    "Cannot determine project root directory."
+
 type M env m =
   (MonadIO m
   ,MonadReader env m
