@@ -80,13 +80,51 @@ benchOptsParser = BenchmarkOpts
 buildOptsParser :: Command
                 -> Parser BuildOpts
 buildOptsParser cmd =
+            transform <$> trace <*> profile <*> options
+  where transform tracing profiling = enable
+          where enable opts
+                  | tracing || profiling =
+                      opts {boptsLibProfile = True
+                           ,boptsExeProfile = True
+                           ,boptsGhcOptions = ["-auto-all","-caf-all"]
+                           ,boptsBenchmarkOpts =
+                                bopts {beoAdditionalArgs =
+                                           beoAdditionalArgs bopts <>
+                                           Just (unwords additionalArgs)}
+                           ,boptsTestOpts =
+                                topts {toAdditionalArgs =
+                                           (toAdditionalArgs topts) <>
+                                           additionalArgs}}
+                  | otherwise = opts
+                  where bopts = boptsBenchmarkOpts opts
+                        topts = boptsTestOpts opts
+                        additionalArgs = [" +RTS ", trac, prof]
+                        trac = if tracing
+                                  then " -xc "
+                                  else ""
+                        prof = if profiling
+                                  then " -p "
+                                  else ""
+        profile =
+            flag False True
+             ( long "profile"
+            <> help "Enable profiling in libraries, executables, etc. \
+                    \for all expressions and generate a profiling report\
+                    \ in exec or benchmarks")
+        trace =
+            flag False True
+             ( long "trace"
+            <> help "Enable profiling in libraries, executables, etc. \
+                    \for all expressions and generate a backtrace on \
+                    \exception")
+        options =
             BuildOpts <$> target <*> libProfiling <*> exeProfiling <*>
             haddock <*> haddockDeps <*> dryRun <*> ghcOpts <*>
             flags <*> copyBins <*> preFetch <*> buildSubset <*>
             fileWatch' <*> keepGoing <*> forceDirty <*> tests <*>
             testOptsParser <*> benches <*> benchOptsParser <*>
             many exec <*> onlyConfigure <*> reconfigure <*> cabalVerbose
-  where target =
+        target =
            many (textArgument
                    (metavar "TARGET" <>
                     help "If none specified, use all packages"))
@@ -320,22 +358,40 @@ configOptsParser hide0 =
   where hide = hideMods hide0
 
 nixOptsParser :: Bool -> Parser NixOptsMonoid
-nixOptsParser hide0 =
-  NixOptsMonoid
+nixOptsParser hide0 = overrideActivation <$>
+  (NixOptsMonoid
   <$> pure False
   <*> maybeBoolFlags nixCmdName
-                     "using a Nix-shell"
+                     "use of a Nix-shell"
                      hide
-  <*> pure []
-  <*> pure Nothing
-  <*> ((map T.pack . fromMaybe [])
+  <*> maybeBoolFlags "nix-pure"
+                     "use of a pure Nix-shell"
+                     hide
+  <*> (fmap (map T.pack)
+       <$> optional (argsOption (long "nix-packages" <>
+                                 metavar "NAMES" <>
+                                 help "List of packages that should be available in the nix-shell (space separated)" <>
+                                 hide)))
+  <*> optional (option str (long "nix-shell-file" <>
+                            metavar "FILEPATH" <>
+                            help "Nix file to be used to launch a nix-shell (for regular Nix users)" <>
+                            hide))
+  <*> (fmap (map T.pack)
        <$> optional (argsOption (long "nix-shell-options" <>
-                                 metavar "OPTION" <>
+                                 metavar "OPTIONS" <>
                                  help "Additional options passed to nix-shell" <>
                                  hide)))
+  <*> (fmap (map T.pack)
+       <$> optional (argsOption (long "nix-path" <>
+                                 metavar "PATH_OPTIONS" <>
+                                 help "Additional options to override NIX_PATH parts (notably 'nixpkgs')" <>
+                                 hide)))
+  )
   where
     hide = hideMods hide0
-
+    overrideActivation m =
+      if m /= mempty then m { nixMonoidEnable = Just True }
+      else m
 
 -- | Options parser configuration for Docker.
 dockerOptsParser :: Bool -> Parser DockerOptsMonoid
@@ -397,7 +453,7 @@ dockerOptsParser hide0 =
                         hide <>
                         metavar "PATH" <>
                         help "Location of image usage tracking database")
-    <*> optional (option str
+    <*> maybeStrOption
             (long(dockerOptName dockerStackExeArgName) <>
              hide <>
              metavar (intercalate "|"
@@ -407,7 +463,7 @@ dockerOptsParser hide0 =
                           , "PATH" ]) <>
              help (concat [ "Location of "
                           , stackProgName
-                          , " executable used in container" ])))
+                          , " executable used in container" ]))
     <*> maybeBoolFlags (dockerOptName dockerSetUserArgName)
                        "setting user in container to match host"
                        hide
@@ -513,6 +569,7 @@ ghciOptsParser = GhciOpts
                                  \module to load, such as for an executable for \
                                  \test suite or benchmark."))
              <*> switch (long "skip-intermediate-deps" <> help "Skip loading intermediate target dependencies")
+             <*> boolFlags True "package-hiding" "package hiding" idm
              <*> buildOptsParser Build
 
 -- | Parser for exec command
@@ -565,12 +622,12 @@ execOptsExtraParser = eoPlainParser <|>
                            help "Use an unmodified environment (only useful with Docker)")
 
 -- | Parser for global command-line options.
-globalOptsParser :: Bool -> Parser GlobalOptsMonoid
-globalOptsParser hide0 =
+globalOptsParser :: Bool -> Maybe LogLevel -> Parser GlobalOptsMonoid
+globalOptsParser hide0 defLogLevel =
     GlobalOptsMonoid <$>
     optional (strOption (long Docker.reExecArgName <> hidden <> internal)) <*>
     optional (option auto (long dockerEntrypointArgName <> hidden <> internal)) <*>
-    logLevelOptsParser hide0 <*>
+    logLevelOptsParser hide0 defLogLevel <*>
     configOptsParser hide0 <*>
     optional (abstractResolverOptsParser hide0) <*>
     optional (compilerOptsParser hide0) <*>
@@ -629,8 +686,8 @@ initOptsParser =
          help "Use the given resolver, even if not all dependencies are met")
 
 -- | Parser for a logging level.
-logLevelOptsParser :: Bool -> Parser (Maybe LogLevel)
-logLevelOptsParser hide =
+logLevelOptsParser :: Bool -> Maybe LogLevel -> Parser (Maybe LogLevel)
+logLevelOptsParser hide defLogLevel =
   fmap (Just . parse)
        (strOption (long "verbosity" <>
                    metavar "VERBOSITY" <>
@@ -640,7 +697,7 @@ logLevelOptsParser hide =
        (short 'v' <> long "verbose" <>
         help ("Enable verbose mode: verbosity level \"" <> showLevel verboseLevel <> "\"") <>
         hideMods hide) <|>
-  pure Nothing
+  pure defLogLevel
   where verboseLevel = LevelDebug
         showLevel l =
           case l of
@@ -746,11 +803,11 @@ newOptsParser = (,) <$> newOpts <*> initOptsParser
         switch
             (long "bare" <>
              help "Do not create a subdirectory for the project") <*>
-        templateNameArgument
+        optional (templateNameArgument
             (metavar "TEMPLATE_NAME" <>
-             help "Name of a template or a local template in a subdirectory,\
-                  \ for example: foo or foo.hsfiles" <>
-             value defaultTemplateName) <*>
+             help "Name of a template or a local template in a file or a URL.\
+                  \ For example: foo or foo.hsfiles or ~/foo or\
+                  \ https://example.com/foo.hsfiles")) <*>
         fmap
             M.fromList
             (many
