@@ -316,7 +316,8 @@ generateBuildInfoOpts sourceMap installedMap mcabalMacros cabalDir distDir omitP
         --
         -- See https://github.com/commercialhaskell/stack/issues/1255
         , bioOneWordOpts = nubOrd $ concat
-            [extOpts b, srcOpts, includeOpts, deps, extra b, extraDirs, fworks b, cObjectFiles]
+            [extOpts b, srcOpts, includeOpts, extra b, extraDirs, fworks b, cObjectFiles]
+        , bioPackageFlags = deps
         , bioCabalMacros = mcabalMacros
         }
   where
@@ -348,7 +349,9 @@ generateBuildInfoOpts sourceMap installedMap mcabalMacros cabalDir distDir omitP
             (("-i" <>) . toFilePathNoTrailingSep)
             ([cabalDir | null (hsSourceDirs b)] <>
              mapMaybe toIncludeDir (hsSourceDirs b) <>
-             [autogenDir distDir,buildDir distDir]) ++
+             [autogenDir distDir,buildDir distDir] <>
+             [makeGenDir (buildDir distDir)
+             | Just makeGenDir <- [fileGenDirFromComponentName componentName]]) ++
         ["-stubdir=" ++ toFilePathNoTrailingSep (buildDir distDir)]
     toIncludeDir "." = Just cabalDir
     toIncludeDir x = fmap (cabalDir </>) (parseRelDir x)
@@ -410,18 +413,22 @@ makeObjectFilePathFromC cabalDir namedComponent distDir cFilePath = do
     relCFilePath <- stripDir cabalDir cFilePath
     relOFilePath <-
         parseRelFile (replaceExtension (toFilePath relCFilePath) "o")
-    addComponentPrefix <- fromComponentName
+    addComponentPrefix <- fileGenDirFromComponentName namedComponent
     return (addComponentPrefix (buildDir distDir) </> relOFilePath)
-  where
-    fromComponentName =
-        case namedComponent of
-            CLib -> return id
-            CExe name -> makeTmp name
-            CTest name -> makeTmp name
-            CBench name -> makeTmp name
-    makeTmp name = do
-        prefix <- parseRelDir (T.unpack name <> "/" <> T.unpack name <> "-tmp")
-        return (</> prefix)
+
+-- | The directory where generated files are put like .o or .hs (from .x files).
+fileGenDirFromComponentName
+    :: MonadThrow m
+    => NamedComponent -> m (Path b Dir -> Path b Dir)
+fileGenDirFromComponentName namedComponent =
+    case namedComponent of
+        CLib -> return id
+        CExe name -> makeTmp name
+        CTest name -> makeTmp name
+        CBench name -> makeTmp name
+  where makeTmp name = do
+            prefix <- parseRelDir (T.unpack name <> "/" <> T.unpack name <> "-tmp")
+            return (</> prefix)
 
 -- | Make the autogen dir.
 autogenDir :: Path Abs Dir -> Path Abs Dir
@@ -445,10 +452,10 @@ packageDependencies =
   allBuildInfo'
 
 -- | Get all build tool dependencies of the package (buildable targets only).
-packageToolDependencies :: PackageDescription -> Map BS.ByteString VersionRange
+packageToolDependencies :: PackageDescription -> Map Text VersionRange
 packageToolDependencies =
   M.fromList .
-  concatMap (fmap (packageNameByteString . depName &&& depRange) .
+  concatMap (fmap (packageNameText . depName &&& depRange) .
              buildTools) .
   allBuildInfo'
 
