@@ -18,7 +18,10 @@ import           Control.Monad.IO.Class
 import           Control.Monad.Logger
 import           Control.Monad.Reader (ask, asks, runReaderT)
 import           Control.Monad.Trans.Control (MonadBaseControl)
-import           Data.Attoparsec.Args (withInterpreterArgs, parseArgs, EscapingMode (Escaping))
+import           Control.Monad.Trans.Either (EitherT)
+import           Control.Monad.Trans.Writer (Writer)
+import           Data.Attoparsec.Args (parseArgs, EscapingMode (Escaping))
+import           Data.Attoparsec.Interpreter (getInterpreterArgs)
 import qualified Data.ByteString.Lazy as L
 import           Data.IORef
 import           Data.List
@@ -43,12 +46,13 @@ import           GHC.IO.Encoding (mkTextEncoding, textEncodingName)
 import           Network.HTTP.Client
 import           Options.Applicative
 import           Options.Applicative.Args
+import           Options.Applicative.Help (errorHelp, stringChunk, vcatChunks)
 import           Options.Applicative.Builder.Extra
 import           Options.Applicative.Complicated
 #ifdef USE_GIT_INFO
 import           Options.Applicative.Simple (simpleVersion)
 #endif
-import           Options.Applicative.Types (readerAsk)
+import           Options.Applicative.Types (readerAsk, ParserHelp(..))
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO
@@ -86,10 +90,10 @@ import           Stack.Upgrade
 import qualified Stack.Upload as Upload
 import           System.Directory (canonicalizePath, doesFileExist, doesDirectoryExist, createDirectoryIfMissing)
 import qualified System.Directory as Directory (findExecutable)
-import           System.Environment (getEnvironment, getProgName)
+import           System.Environment (getEnvironment, getProgName, getArgs, withArgs)
 import           System.Exit
 import           System.FileLock (lockFile, tryLockFile, unlockFile, SharedExclusive(Exclusive), FileLock)
-import           System.FilePath (searchPathSeparator)
+import           System.FilePath (pathSeparator, searchPathSeparator)
 import           System.IO (hIsTerminalDevice, stderr, stdin, stdout, hSetBuffering, BufferMode(..), hPutStrLn, Handle, hGetEncoding, hSetEncoding)
 import           System.Process.Read
 
@@ -105,363 +109,421 @@ hSetTranslit h = do
               hSetEncoding h enc'
         _ -> return ()
 
--- | Commandline dispatcher.
-main :: IO ()
-main = withInterpreterArgs stackProgName $ \args isInterpreter -> do
-     -- Line buffer the output by default, particularly for non-terminal runs.
-     -- See https://github.com/commercialhaskell/stack/pull/360
-     hSetBuffering stdout LineBuffering
-     hSetBuffering stdin  LineBuffering
-     hSetBuffering stderr LineBuffering
-     hSetTranslit stdout
-     hSetTranslit stderr
-     progName <- getProgName
-     isTerminal <- hIsTerminalDevice stdout
-     execExtraHelp args
-                   dockerHelpOptName
-                   (dockerOptsParser False)
-                   ("Only showing --" ++ Docker.dockerCmdName ++ "* options.")
-     execExtraHelp args
-                   nixHelpOptName
-                   (nixOptsParser False)
-                   ("Only showing --" ++ Nix.nixCmdName ++ "* options.")
+versionString' :: String
 #ifdef USE_GIT_INFO
-     let commitCount = $gitCommitCount
-         versionString' = concat $ concat
-            [ [$(simpleVersion Meta.version)]
-              -- Leave out number of commits for --depth=1 clone
-              -- See https://github.com/commercialhaskell/stack/issues/792
-            , [" (" ++ commitCount ++ " commits)" | commitCount /= ("1"::String) &&
-                                                    commitCount /= ("UNKNOWN" :: String)]
-            , [" ", display buildArch]
-            ]
+versionString' = concat $ concat
+    [ [$(simpleVersion Meta.version)]
+      -- Leave out number of commits for --depth=1 clone
+      -- See https://github.com/commercialhaskell/stack/issues/792
+    , [" (" ++ commitCount ++ " commits)" | commitCount /= ("1"::String) &&
+                                          commitCount /= ("UNKNOWN" :: String)]
+    , [" ", display buildArch]
+    ]
+    where commitCount = $gitCommitCount
 #else
-     let versionString' = showVersion Meta.version ++ ' ' : display buildArch
+versionString' = showVersion Meta.version ++ ' ' : display buildArch
 #endif
 
-     let globalOpts hide =
-             extraHelpOption hide progName (Docker.dockerCmdName ++ "*") dockerHelpOptName <*>
-             extraHelpOption hide progName (Nix.nixCmdName ++ "*") nixHelpOptName <*>             
-             globalOptsParser hide
-         addCommand' cmd title footerStr constr =
-             addCommand cmd title footerStr constr (globalOpts True)
-         addSubCommands' cmd title footerStr =
-             addSubCommands cmd title footerStr (globalOpts True)
-     eGlobalRun <- try $
-       complicatedOptions
-         Meta.version
-         (Just versionString')
-         "stack - The Haskell Tool Stack"
-         ""
-         (globalOpts False)
-         -- when there's a parse failure
-         (Just $ \f as ->
-           -- fall-through to external executables in `git` style if they exist
-           -- (i.e. `stack something` looks for `stack-something` before
-           -- failing with "Invalid argument `something'")
-           case stripPrefix "Invalid argument" (fst (renderFailure f "")) of
-             Just _ -> do
-               mExternalExec <- Directory.findExecutable ("stack-" ++ head as)
-               case mExternalExec of
-                 Just ex -> do
-                   menv <- getEnvOverride buildPlatform
-                   runNoLoggingT (exec menv ex (tail as))
-                 Nothing -> handleParseResult (Failure f)
-             Nothing -> handleParseResult (Failure f)
-         )
-         (do addCommand' "build"
-                        "Build the package(s) in this directory/configuration"
-                        cmdFooter
-                        buildCmd
-                        (buildOptsParser Build)
-             addCommand' "install"
-                        "Shortcut for 'build --copy-bins'"
-                        cmdFooter
-                        buildCmd
-                        (buildOptsParser Install)
-             addCommand' "uninstall"
-                        "DEPRECATED: This command performs no actions, and is present for documentation only"
-                        cmdFooter
-                        uninstallCmd
-                        (many $ strArgument $ metavar "IGNORED")
-             addCommand' "test"
-                        "Shortcut for 'build --test'"
-                        cmdFooter
-                        buildCmd
-                        (buildOptsParser Test)
-             addCommand' "bench"
-                        "Shortcut for 'build --bench'"
-                        cmdFooter
-                        buildCmd
-                        (buildOptsParser Bench)
-             addCommand' "haddock"
-                        "Shortcut for 'build --haddock'"
-                        cmdFooter
-                        buildCmd
-                        (buildOptsParser Haddock)
-             addCommand' "new"
-                        "Create a new project from a template. Run `stack templates' to see available templates."
-                        cmdFooter
-                        newCmd
-                        newOptsParser
-             addCommand' "templates"
-                        "List the templates available for `stack new'."
-                        cmdFooter
-                        templatesCmd
-                        (pure ())
-             addCommand' "init"
-                        "Initialize a stack project based on one or more cabal packages"
-                        cmdFooter
-                        initCmd
-                        initOptsParser
-             addCommand' "solver"
-                        "Use a dependency solver to try and determine missing extra-deps"
-                        cmdFooter
-                        solverCmd
-                        solverOptsParser
-             addCommand' "setup"
-                        "Get the appropriate GHC for your project"
-                        cmdFooter
-                        setupCmd
-                        setupParser
-             addCommand' "path"
-                        "Print out handy path information"
-                        cmdFooter
-                        pathCmd
-                        (mapMaybeA
-                            (\(desc,name,_) ->
-                                 flag Nothing
-                                      (Just name)
-                                      (long (T.unpack name) <>
-                                       help desc))
-                            paths)
-             addCommand' "unpack"
-                        "Unpack one or more packages locally"
-                        cmdFooter
-                        unpackCmd
-                        (some $ strArgument $ metavar "PACKAGE")
-             addCommand' "update"
-                        "Update the package index"
-                        cmdFooter
-                        updateCmd
-                        (pure ())
-             addCommand' "upgrade"
-                        "Upgrade to the latest stack (experimental)"
-                        cmdFooter
-                        upgradeCmd
-                        ((,) <$> switch
-                                  ( long "git"
-                                 <> help "Clone from Git instead of downloading from Hackage (more dangerous)" )
-                             <*> strOption
-                                  ( long "git-repo"
-                                 <> help "Clone from specified git repository"
-                                 <> value "https://github.com/commercialhaskell/stack"
-                                 <> showDefault ))
-             addCommand' "upload"
-                        "Upload a package to Hackage"
-                        cmdFooter
-                        uploadCmd
-                        ((,,,)
-                         <$> many (strArgument $ metavar "TARBALL/DIR")
-                         <*> optional pvpBoundsOption
-                         <*> ignoreCheckSwitch
-                         <*> flag False True
-                              (long "sign" <>
-                               help "GPG sign & submit signature"))
-             addCommand' "sdist"
-                        "Create source distribution tarballs"
-                        cmdFooter
-                        sdistCmd
-                        ((,,)
-                         <$> many (strArgument $ metavar "DIR")
-                         <*> optional pvpBoundsOption
-                         <*> ignoreCheckSwitch)
-             addCommand' "dot"
-                        "Visualize your project's dependency graph using Graphviz dot"
-                        cmdFooter
-                        dotCmd
-                        dotOptsParser
-             addCommand' "exec"
-                        "Execute a command"
-                        cmdFooter
-                        execCmd
-                        (execOptsParser Nothing)
-             addCommand' "ghc"
-                        "Run ghc"
-                        cmdFooter
-                        execCmd
-                        (execOptsParser $ Just ExecGhc)
-             addCommand' "ghci"
-                        "Run ghci in the context of package(s) (experimental)"
-                        cmdFooter
-                        ghciCmd
-                        ghciOptsParser
-             addCommand' "repl"
-                        "Run ghci in the context of package(s) (experimental) (alias for 'ghci')"
-                        cmdFooter
-                        ghciCmd
-                        ghciOptsParser
-             addCommand' "runghc"
-                        "Run runghc"
-                        cmdFooter
-                        execCmd
-                        (execOptsParser $ Just ExecRunGhc)
-             addCommand' "runhaskell"
-                        "Run runghc (alias for 'runghc')"
-                        cmdFooter
-                        execCmd
-                        (execOptsParser $ Just ExecRunGhc)
-             addCommand' "eval"
-                        "Evaluate some haskell code inline. Shortcut for 'stack exec ghc -- -e CODE'"
-                        cmdFooter
-                        evalCmd
-                        (evalOptsParser "CODE")
-             addCommand' "clean"
-                        "Clean the local packages"
-                        cmdFooter
-                        cleanCmd
-                        cleanOptsParser
-             addCommand' "list-dependencies"
-                        "List the dependencies"
-                        cmdFooter
-                        listDependenciesCmd
-                        (textOption (long "separator" <>
-                                     metavar "SEP" <>
-                                     help ("Separator between package name " <>
-                                           "and package version.") <>
-                                     value " " <>
-                                     showDefault))
-             addCommand' "query"
-                        "Query general build information (experimental)"
-                        cmdFooter
-                        queryCmd
-                        (many $ strArgument $ metavar "SELECTOR...")
-             addSubCommands'
-                 "ide"
-                 "IDE-specific commands"
-                 cmdFooter
-                 (do addCommand'
-                         "start"
-                         "Start the ide-backend service"
-                         cmdFooter
-                         ideCmd
-                         ((,) <$> many (textArgument
-                                          (metavar "TARGET" <>
-                                           help ("If none specified, use all " <>
-                                                 "packages defined in current directory")))
-                              <*> argsOption (long "ghc-options" <>
-                                              metavar "OPTION" <>
-                                              help "Additional options passed to GHCi" <>
-                                              value []))
-                     addCommand'
-                         "packages"
-                         "List all available local loadable packages"
-                         cmdFooter
-                         packagesCmd
-                         (pure ())
-                     addCommand'
-                         "load-targets"
-                         "List all load targets for a package target"
-                         cmdFooter
-                         targetsCmd
-                         (textArgument
-                            (metavar "TARGET")))
-             addSubCommands'
-               Docker.dockerCmdName
-               "Subcommands specific to Docker use"
-               cmdFooter
-               (do addCommand' Docker.dockerPullCmdName
-                              "Pull latest version of Docker image from registry"
-                              cmdFooter
-                              dockerPullCmd
-                              (pure ())
-                   addCommand' "reset"
-                              "Reset the Docker sandbox"
-                              cmdFooter
-                              dockerResetCmd
-                              (switch (long "keep-home" <>
-                                       help "Do not delete sandbox's home directory"))
-                   addCommand' Docker.dockerCleanupCmdName
-                              "Clean up Docker images and containers"
-                              cmdFooter
-                              dockerCleanupCmd
-                              dockerCleanupOptsParser)
-             addSubCommands'
-                ConfigCmd.cfgCmdName
-                "Subcommands specific to modifying stack.yaml files"
-                cmdFooter
-                (addCommand' ConfigCmd.cfgCmdSetName
-                            "Sets a field in the project's stack.yaml to value"
-                            cmdFooter
-                            cfgSetCmd
-                            configCmdSetParser)
-             addSubCommands'
-               Image.imgCmdName
-               "Subcommands specific to imaging (EXPERIMENTAL)"
-               cmdFooter
-               (addCommand' Image.imgDockerCmdName
-                "Build a Docker image for the project"
-                cmdFooter
-                imgDockerCmd
-                (boolFlags True
-                    "build"
-                    "building the project before creating the container"
-                    idm))
-             addSubCommands'
-               "hpc"
-               "Subcommands specific to Haskell Program Coverage"
-               cmdFooter
-               (addCommand' "report"
-                            "Generate HPC report a combined HPC report"
-                            cmdFooter
-                            hpcReportCmd
-                            hpcReportOptsParser)
-             addSubCommands'
-               Sig.sigCmdName
-               "Subcommands specific to package signatures (EXPERIMENTAL)"
-               cmdFooter
-               (addSubCommands'
-                  Sig.sigSignCmdName
-                  "Sign a a single package or all your packages"
-                  cmdFooter
-                  (addCommand'
-                     Sig.sigSignSdistCmdName
-                     "Sign a single sdist package file"
-                     cmdFooter
-                     sigSignSdistCmd
-                     Sig.sigSignSdistOpts)))
-     case eGlobalRun of
-       Left (exitCode :: ExitCode) -> do
-         when isInterpreter $
-           hPutStrLn stderr $ concat
-             [ "\nIf you are trying to use "
-             , stackProgName
-             , " as a script interpreter, a\n'-- "
-             , stackProgName
-             , " [options] runghc [options]' comment is required."
-             , "\nSee https://github.com/commercialhaskell/stack/blob/release/doc/GUIDE.md#ghcrunghc" ]
-         throwIO exitCode
-       Right (globalMonoid,run) -> do
-         let global = globalOptsFromMonoid isTerminal globalMonoid
-         when (globalLogLevel global == LevelDebug) $ hPutStrLn stderr versionString'
-         case globalReExecVersion global of
-             Just expectVersion
-                 | expectVersion /= showVersion Meta.version ->
-                     throwIO $ InvalidReExecVersion expectVersion (showVersion Meta.version)
-             _ -> return ()
-         run global `catch` \e ->
-            -- This special handler stops "stack: " from being printed before the
-            -- exception
-            case fromException e of
-                Just ec -> exitWith ec
-                Nothing -> do
-                    printExceptionStderr e
-                    exitFailure
+main :: IO ()
+main = do
+  -- Line buffer the output by default, particularly for non-terminal runs.
+  -- See https://github.com/commercialhaskell/stack/pull/360
+  hSetBuffering stdout LineBuffering
+  hSetBuffering stdin  LineBuffering
+  hSetBuffering stderr LineBuffering
+  hSetTranslit stdout
+  hSetTranslit stderr
+  args <- getArgs
+  progName <- getProgName
+  isTerminal <- hIsTerminalDevice stdout
+  execExtraHelp args
+                Docker.dockerHelpOptName
+                (dockerOptsParser False)
+                ("Only showing --" ++ Docker.dockerCmdName ++ "* options.")
+  execExtraHelp args
+                Nix.nixHelpOptName
+                (nixOptsParser False)
+                ("Only showing --" ++ Nix.nixCmdName ++ "* options.")
+
+  eGlobalRun <- try $ commandLineHandler progName False
+  case eGlobalRun of
+    Left (exitCode :: ExitCode) -> do
+      throwIO exitCode
+    Right (globalMonoid,run) -> do
+      let global = globalOptsFromMonoid isTerminal globalMonoid
+      when (globalLogLevel global == LevelDebug) $ hPutStrLn stderr versionString'
+      case globalReExecVersion global of
+          Just expectVersion
+              | expectVersion /= showVersion Meta.version ->
+                  throwIO $ InvalidReExecVersion expectVersion (showVersion Meta.version)
+          _ -> return ()
+      run global `catch` \e ->
+          -- This special handler stops "stack: " from being printed before the
+          -- exception
+          case fromException e of
+              Just ec -> exitWith ec
+              Nothing -> do
+                  printExceptionStderr e
+                  exitFailure
+
+-- Vertically combine only the error component of the first argument with the
+-- error component of the second.
+vcatErrorHelp :: ParserHelp -> ParserHelp -> ParserHelp
+vcatErrorHelp (ParserHelp e1 _ _ _ _) (ParserHelp e2 h2 u2 b2 f2) =
+  ParserHelp (vcatChunks [e2, e1]) h2 u2 b2 f2
+
+commandLineHandler
+  :: String
+  -> Bool
+  -> IO (GlobalOptsMonoid, GlobalOpts -> IO ())
+commandLineHandler progName isInterpreter = complicatedOptions
+  Meta.version
+  (Just versionString')
+  "stack - The Haskell Tool Stack"
+  ""
+  (globalOpts False)
+  (Just failureCallback)
+  (addCommands (globalOpts True) isInterpreter)
   where
-    ignoreCheckSwitch = switch (long "ignore-check" <> help "Do not check package for common mistakes")
-    dockerHelpOptName = Docker.dockerCmdName ++ "-help"
-    nixHelpOptName    = Nix.nixCmdName ++ "-help"
-    cmdFooter = "Run 'stack --help' for global options that apply to all subcommands."
+    failureCallback f args =
+      case stripPrefix "Invalid argument" (fst (renderFailure f "")) of
+          Just _ -> if isInterpreter
+                    then parseResultHandler args f
+                    else secondaryCommandHandler args f
+                        >>= interpreterHandler args
+          Nothing -> parseResultHandler args f
+
+    parseResultHandler args f =
+      if isInterpreter
+      then do
+        let hlp = errorHelp $ stringChunk
+              (unwords ["Error executing interpreter command:"
+                        , progName
+                        , unwords args])
+        handleParseResult (overFailure (vcatErrorHelp hlp) (Failure f))
+      else handleParseResult (Failure f)
+
+    globalOpts hide =
+      extraHelpOption hide progName (Docker.dockerCmdName ++ "*") Docker.dockerHelpOptName <*>
+      extraHelpOption hide progName (Nix.nixCmdName ++ "*") Nix.nixHelpOptName <*>
+      globalOptsParser hide (if isInterpreter
+                             then Just $ LevelOther "silent"
+                             else Nothing)
+
+globalFooter :: String
+globalFooter = "Run 'stack --help' for global options that apply to all subcommands."
+
+addCommands
+  :: Monoid c
+  => Parser c
+  -> Bool
+  -> EitherT (GlobalOpts -> IO ())
+             (Writer (Mod CommandFields (GlobalOpts -> IO (), c)))
+             ()
+addCommands globalOpts isInterpreter = do
+  when (not isInterpreter) (do
+    addCommand' "build"
+                "Build the package(s) in this directory/configuration"
+                buildCmd
+                (buildOptsParser Build)
+    addCommand' "install"
+                "Shortcut for 'build --copy-bins'"
+                buildCmd
+                (buildOptsParser Install)
+    addCommand' "uninstall"
+                "DEPRECATED: This command performs no actions, and is present for documentation only"
+                uninstallCmd
+                (many $ strArgument $ metavar "IGNORED")
+    addCommand' "test"
+                "Shortcut for 'build --test'"
+                buildCmd
+                (buildOptsParser Test)
+    addCommand' "bench"
+                "Shortcut for 'build --bench'"
+                buildCmd
+                (buildOptsParser Bench)
+    addCommand' "haddock"
+                "Shortcut for 'build --haddock'"
+                buildCmd
+                (buildOptsParser Haddock)
+    addCommand' "new"
+                "Create a new project from a template. Run `stack templates' to see available templates."
+                newCmd
+                newOptsParser
+    addCommand' "templates"
+                "List the templates available for `stack new'."
+                templatesCmd
+                (pure ())
+    addCommand' "init"
+                "Initialize a stack project based on one or more cabal packages"
+                initCmd
+                initOptsParser
+    addCommand' "solver"
+                "Use a dependency solver to try and determine missing extra-deps"
+                solverCmd
+                solverOptsParser
+    addCommand' "setup"
+                "Get the appropriate GHC for your project"
+                setupCmd
+                setupParser
+    addCommand' "path"
+                "Print out handy path information"
+                pathCmd
+                (mapMaybeA
+                    (\(desc,name,_) ->
+                         flag Nothing
+                              (Just name)
+                              (long (T.unpack name) <>
+                               help desc))
+                    paths)
+    addCommand' "unpack"
+                "Unpack one or more packages locally"
+                unpackCmd
+                (some $ strArgument $ metavar "PACKAGE")
+    addCommand' "update"
+                "Update the package index"
+                updateCmd
+                (pure ())
+    addCommand' "upgrade"
+                "Upgrade to the latest stack (experimental)"
+                upgradeCmd
+                ((,) <$> switch
+                          ( long "git"
+                         <> help "Clone from Git instead of downloading from Hackage (more dangerous)" )
+                     <*> strOption
+                          ( long "git-repo"
+                         <> help "Clone from specified git repository"
+                         <> value "https://github.com/commercialhaskell/stack"
+                         <> showDefault ))
+    addCommand' "upload"
+                "Upload a package to Hackage"
+                uploadCmd
+                ((,,,)
+                 <$> many (strArgument $ metavar "TARBALL/DIR")
+                 <*> optional pvpBoundsOption
+                 <*> ignoreCheckSwitch
+                 <*> flag False True
+                      (long "sign" <>
+                       help "GPG sign & submit signature"))
+    addCommand' "sdist"
+                "Create source distribution tarballs"
+                sdistCmd
+                ((,,)
+                 <$> many (strArgument $ metavar "DIR")
+                 <*> optional pvpBoundsOption
+                 <*> ignoreCheckSwitch)
+    addCommand' "dot"
+                "Visualize your project's dependency graph using Graphviz dot"
+                dotCmd
+                dotOptsParser
+    addCommand' "exec"
+                "Execute a command"
+                execCmd
+                (execOptsParser Nothing)
+    addCommand' "ghc"
+                "Run ghc"
+                execCmd
+                (execOptsParser $ Just ExecGhc)
+    addCommand' "ghci"
+                "Run ghci in the context of package(s) (experimental)"
+                ghciCmd
+                ghciOptsParser
+    addCommand' "repl"
+                "Run ghci in the context of package(s) (experimental) (alias for 'ghci')"
+                ghciCmd
+                ghciOptsParser
+    )
+
+  -- These two are the only commands allowed in interpreter mode as well
+  addCommand' "runghc"
+              "Run runghc"
+              execCmd
+              (execOptsParser $ Just ExecRunGhc)
+  addCommand' "runhaskell"
+              "Run runghc (alias for 'runghc')"
+              execCmd
+              (execOptsParser $ Just ExecRunGhc)
+
+  when (not isInterpreter) (do
+    addCommand' "eval"
+                "Evaluate some haskell code inline. Shortcut for 'stack exec ghc -- -e CODE'"
+                evalCmd
+                (evalOptsParser "CODE")
+    addCommand' "clean"
+                "Clean the local packages"
+                cleanCmd
+                cleanOptsParser
+    addCommand' "list-dependencies"
+                "List the dependencies"
+                listDependenciesCmd
+                (textOption (long "separator" <>
+                             metavar "SEP" <>
+                             help ("Separator between package name " <>
+                                   "and package version.") <>
+                             value " " <>
+                             showDefault))
+    addCommand' "query"
+                "Query general build information (experimental)"
+                queryCmd
+                (many $ strArgument $ metavar "SELECTOR...")
+    addSubCommands'
+        "ide"
+        "IDE-specific commands"
+        (do addCommand'
+                "start"
+                "Start the ide-backend service"
+                ideCmd
+                ((,) <$> many (textArgument
+                                  (metavar "TARGET" <>
+                                   help ("If none specified, use all " <>
+                                         "packages defined in current directory")))
+                      <*> argsOption (long "ghc-options" <>
+                                    metavar "OPTION" <>
+                                    help "Additional options passed to GHCi" <>
+                                    value []))
+            addCommand'
+                "packages"
+                "List all available local loadable packages"
+                packagesCmd
+                (pure ())
+            addCommand'
+                "load-targets"
+                "List all load targets for a package target"
+                targetsCmd
+                (textArgument
+                    (metavar "TARGET")))
+    addSubCommands'
+      Docker.dockerCmdName
+      "Subcommands specific to Docker use"
+      (do addCommand' Docker.dockerPullCmdName
+                      "Pull latest version of Docker image from registry"
+                      dockerPullCmd
+                      (pure ())
+          addCommand' "reset"
+                      "Reset the Docker sandbox"
+                      dockerResetCmd
+                      (switch (long "keep-home" <>
+                               help "Do not delete sandbox's home directory"))
+          addCommand' Docker.dockerCleanupCmdName
+                      "Clean up Docker images and containers"
+                      dockerCleanupCmd
+                      dockerCleanupOptsParser)
+    addSubCommands'
+        ConfigCmd.cfgCmdName
+        "Subcommands specific to modifying stack.yaml files"
+        (addCommand' ConfigCmd.cfgCmdSetName
+                    "Sets a field in the project's stack.yaml to value"
+                    cfgSetCmd
+                    configCmdSetParser)
+    addSubCommands'
+      Image.imgCmdName
+      "Subcommands specific to imaging (EXPERIMENTAL)"
+      (addCommand' Image.imgDockerCmdName
+        "Build a Docker image for the project"
+        imgDockerCmd
+        (boolFlags True
+            "build"
+            "building the project before creating the container"
+            idm))
+    addSubCommands'
+      "hpc"
+      "Subcommands specific to Haskell Program Coverage"
+      (addCommand' "report"
+                    "Generate HPC report a combined HPC report"
+                    hpcReportCmd
+                    hpcReportOptsParser)
+    addSubCommands'
+      Sig.sigCmdName
+      "Subcommands specific to package signatures (EXPERIMENTAL)"
+      (addSubCommands'
+          Sig.sigSignCmdName
+          "Sign a a single package or all your packages"
+          (addCommand'
+            Sig.sigSignSdistCmdName
+            "Sign a single sdist package file"
+            sigSignSdistCmd
+            Sig.sigSignSdistOpts))
+    )
+  where
+    ignoreCheckSwitch =
+        switch (long "ignore-check"
+                <> help "Do not check package for common mistakes")
+
+    -- addCommand hiding global options
+    addCommand' cmd title constr =
+        addCommand cmd title globalFooter constr globalOpts
+
+    addSubCommands' cmd title =
+        addSubCommands cmd title globalFooter globalOpts
+
+secondaryCommandHandler
+  :: [String]
+  -> ParserFailure ParserHelp
+  -> IO (ParserFailure ParserHelp)
+
+-- fall-through to external executables in `git` style if they exist
+-- (i.e. `stack something` looks for `stack-something` before
+-- failing with "Invalid argument `something'")
+secondaryCommandHandler args f =
+    -- don't even try when the argument looks like a path
+    if elem pathSeparator cmd
+       then return f
+    else do
+      mExternalExec <- Directory.findExecutable cmd
+      case mExternalExec of
+        Just ex -> do
+          menv <- getEnvOverride buildPlatform
+          -- TODO show the command in verbose mode
+          -- hPutStrLn stderr $ unwords $
+          --   ["Running", "[" ++ ex, unwords (tail args) ++ "]"]
+          _ <- runNoLoggingT (exec menv ex (tail args))
+          return f
+        Nothing -> return $ fmap (vcatErrorHelp (noSuchCmd cmd)) f
+  where
+    -- FIXME this is broken when any options are specified before the command
+    -- e.g. stack --verbosity silent cmd
+    cmd = stackProgName ++ "-" ++ (head args)
+    noSuchCmd name = errorHelp $ stringChunk
+      ("Auxiliary command not found in path `" ++ name ++ "'")
+
+interpreterHandler
+  :: Monoid t
+  => [String]
+  -> ParserFailure ParserHelp
+  -> IO (GlobalOptsMonoid, (GlobalOpts -> IO (), t))
+interpreterHandler args f = do
+  isFile <- doesFileExist file
+  if isFile
+  then runInterpreterCommand file
+  else parseResultHandler (errorCombine (noSuchFile file))
+  where
+    file = head args
+
+    -- if the filename contains a path separator then we know that it is not a
+    -- command it is a file to be interpreted. In that case we only show the
+    -- interpreter error message and exclude the command related error messages.
+    errorCombine =
+      if elem pathSeparator file
+      then overrideErrorHelp
+      else vcatErrorHelp
+
+    overrideErrorHelp (ParserHelp e1 _ _ _ _) (ParserHelp _ h2 u2 b2 f2) =
+      ParserHelp e1 h2 u2 b2 f2
+
+    parseResultHandler fn = handleParseResult (overFailure fn (Failure f))
+    noSuchFile name = errorHelp $ stringChunk
+      ("File does not exist or is not a regular file `" ++ name ++ "'")
+
+    runInterpreterCommand path = do
+      progName <- getProgName
+      iargs <- getInterpreterArgs path
+      let parseCmdLine = commandLineHandler progName True
+      let cmdArgs = iargs ++ "--" : args
+       -- TODO show the command in verbose mode
+       -- hPutStrLn stderr $ unwords $
+       --   ["Running", "[" ++ progName, unwords cmdArgs ++ "]"]
+      (a,b) <- withArgs cmdArgs parseCmdLine
+      return (a,(b,mempty))
 
 -- | Print out useful path information in a human-readable format (and
 -- support others later).
@@ -643,7 +705,7 @@ setupCmd SetupCmdOpts{..} go@GlobalOpts{..} = do
           (lcProjectRoot lc)
           Nothing
           (runStackTGlobal manager (lcConfig lc) go $
-           Nix.reexecWithOptionalShell $
+           Nix.reexecWithOptionalShell (lcProjectRoot lc) $
            runStackLoggingTGlobal manager go $ do
               (wantedCompiler, compilerCheck, mstack) <-
                   case scoCompilerVersion of
@@ -809,8 +871,7 @@ withBuildConfigExt go@GlobalOpts{..} mbefore inner mafter = do
                  (lcProjectRoot lc)
                  mbefore
                  (runStackTGlobal manager (lcConfig lc) go $
-                    Nix.reexecWithOptionalShell (inner'' lk0)
-                 )
+                    Nix.reexecWithOptionalShell (lcProjectRoot lc) (inner'' lk0))
                  mafter
                  (Just $ liftIO $
                       do lk' <- readIORef curLk
@@ -952,6 +1013,7 @@ execCmd ExecOpts {..} go@GlobalOpts{..} =
                         config <- asks getConfig
                         menv <- liftIO $ configEnvOverride config plainEnvSettings
                         Nix.reexecWithOptionalShell
+                            (lcProjectRoot lc)
                             (runStackTGlobal manager (lcConfig lc) go $
                                 exec menv cmd args))
                     Nothing
